import time
import slskproto
import slskmessages
import transfers
import Queue
import threading
import images
import about
import userinfobrowse
import search
from wxPython.wx import *

class OutputWindow(wxTextCtrl):
    """
    Multiple-lines text control with a twist - cleans oldest 100 lines from 
    itself when 200 lines maximum is reached
    """
    def __init__(self, parent, id, style):
        wxTextCtrl.__init__(self,parent,id,style = style)
        self.lines = 1
        self.chars = 0

    def AppendText(self, text):
        wxTextCtrl.AppendText(self,text)
        self.lines = self.lines + 1
        if self.lines == 100:
             self.chars = wxTextCtrl.GetLastPosition(self)
        if self.lines == 200:
             wxTextCtrl.Remove(self,0,self.chars)
             wxTextCtrl.SetInsertionPointEnd(self)
             self.chars = wxTextCtrl.GetLastPosition(self)
             self.lines = 100

class ChatRooms(wxSplitterWindow):
    """ This window contains a rooms list and a rooms notebook """
    def __init__(self, parent, id, queue, privatechat, peerconns, frame,  style = wxNO_3D|wxSP_3D):
        wxSplitterWindow.__init__(self,parent,id,wxDefaultPosition,wxDefaultSize,style,"splitterWindow")
        self.roomsnotebook = wxNotebook(self, -1, style=wxCLIP_CHILDREN)
        self.roomsctrl = RoomsCtrl(self,-1, queue, privatechat, peerconns,frame, self.roomsnotebook)
        self.SplitVertically(self.roomsnotebook,self.roomsctrl)
        self.SetSashPosition(610, true)


class RoomsCtrl(wxPanel):
    """ This panel contains a rooms list and an input box to create a room """
    def __init__(self, parent, id, queue, privatechat, peerconns,frame, roomsnotebook):
        wxPanel.__init__(self, parent, id)
        self.roomlist = RoomListCtrl(self,-1)
        self.addroom = wxTextCtrl(self,-1, style = wxTE_PROCESS_ENTER)
        self.create = wxStaticText(self, -1, "Create:")

        sizerh = wxBoxSizer(wxHORIZONTAL)
        sizerh.Add(self.create,0,wxALIGN_CENTER)
        sizerh.Add(self.addroom,1,wxEXPAND)
        sizerv = wxBoxSizer(wxVERTICAL)
        sizerv.Add(self.roomlist,1,wxEXPAND)
        sizerv.Add(sizerh,0,wxEXPAND)

        self.SetSizer(sizerv)
        self.SetAutoLayout(true)
        EVT_TEXT_ENTER(self,self.addroom.GetId(),self.OnEnter)

        self.joinedrooms = {}
        self.joinedroomslist = []
        self.rooms = []
        self.parent = parent
        self.queue = queue
        self.privatechat = privatechat
        self.peerconns = peerconns
        self.frame = frame
        self.roomsnotebook = roomsnotebook

        self.menu = wxMenu()
        self.joinID=wxNewId()
        self.menu.Append(self.joinID, 'Join Room')
        EVT_MENU(self,self.joinID, self.OnJoinRoom)
        self.leaveID=wxNewId()
        self.menu.Append(self.leaveID, 'Leave Room')
        EVT_MENU(self,self.leaveID, self.OnLeaveRoom)
        self.menu.AppendSeparator()
        refreshID=wxNewId()
        self.menu.Append(refreshID, 'Refresh Room List')
        EVT_MENU(self,refreshID, self.OnRefresh)

        EVT_RIGHT_UP(self.roomlist, self.OnRightUp)

    def OnEnter(self, event):
	""" Add a room or join an existing one """
        text = self.addroom.GetLineText(0)
        self.queue.put(slskmessages.JoinRoom(text))
        self.addroom.Clear()

    def OnRightUp(self, event):
        """ Show the pop-up menu"""
        pt = event.GetPosition()
        item, flags = self.roomlist.HitTest(pt)
        self.item = item
        if item != -1:
            self.roomlist.SetItemState(item,wxLIST_STATE_FOCUSED,wxLIST_STATE_FOCUSED)
            if self.rooms[self.item][0] in self.joinedrooms.keys():
                self.menu.Enable(self.joinID,0)
                self.menu.Enable(self.leaveID,1)
            else:
                self.menu.Enable(self.joinID,1)
                self.menu.Enable(self.leaveID,0)
            self.roomlist.PopupMenu(self.menu, pt)

    def OnJoinRoom(self, event):
        self.queue.put(slskmessages.JoinRoom(self.rooms[self.item][0]))

    def OnLeaveRoom(self, event):
        self.queue.put(slskmessages.LeaveRoom(self.rooms[self.item][0]))

    def OnRefresh(self,event):
        self.queue.put(slskmessages.RoomList())

    def SetGreeting(self, text):
# Don't know where to put that at the moment
        self.greeting = text

    def SetRoomList(self, msg):
        self.rooms = msg.rooms
        self.rooms.sort(self.SortRoomList)
        self.roomlist.SetItemCount(len(self.rooms))

    def SortRoomList(self, item1, item2):
        if item1[1] > item2[1]:
            return -1
        elif item1[1] < item2[1]:
            return 1
        else:
            return 0

    def UserJoinedRoom(self, msg):
        self.CreateRoomWindow(msg.room)
        self.joinedrooms[msg.room].UserJoinedRoom(msg)

    def JoinRoom(self,msg):
        self.CreateRoomWindow(msg.room)
        self.joinedrooms[msg.room].JoinRoom(msg)

    def SayChatRoom(self,msg):
        self.CreateRoomWindow(msg.room)
        self.joinedrooms[msg.room].SayChatRoom(msg)

    def UserLeftRoom(self,msg):
        self.CreateRoomWindow(msg.room)
        self.joinedrooms[msg.room].UserLeftRoom(msg)

    def QueuedDownloads(self, msg):
        for i in self.joinedrooms.keys():
            if msg.user in self.joinedrooms[i].users.keys():
                self.joinedrooms[i].QueuedDownloads(msg)

    def GetUserStatus(self, msg):
        for i in self.joinedrooms.keys():
            if msg.user in self.joinedrooms[i].users.keys():
                self.joinedrooms[i].GetUserStatus(msg)

    def GetUserStats(self, msg):
        for i in self.joinedrooms.keys():
            if msg.user in self.joinedrooms[i].users.keys():
                self.joinedrooms[i].GetUserStats(msg)

    def LeaveRoom(self,msg):
        self.roomsnotebook.DeletePage(self.joinedroomslist.index(msg.room))
        self.joinedroomslist.remove(msg.room)
        del self.joinedrooms[msg.room]

    def CreateRoomWindow(self, room):
        if room not in self.joinedrooms.keys():
            self.joinedrooms[room] = ChatWindow(self.roomsnotebook, -1, self.queue, self.privatechat, self.peerconns, self.frame)

            self.roomsnotebook.AddPage(self.joinedrooms[room],room)
            self.joinedroomslist.append(room)

class RoomListCtrl(wxListCtrl):
    """ Actual room list control """
    def __init__(self, parent, id, style = wxLC_REPORT|wxLC_VIRTUAL):
        wxListCtrl.__init__(self,parent,id,style = style)
        self.InsertColumn(0,"Room", width=100)
        self.InsertColumn(1,"Users",width=50)
        self.SetItemCount(0)

        self.parent = parent

    def OnGetItemText(self, item, col):
        return self.parent.rooms[item][col]

class GlobalUsersList(wxPanel):
    """ This window contains a global list of users. """
    def __init__(self, parent, id, queue, privatechat, peerconns,frame):
        wxPanel.__init__(self, parent, id)

	self.queue = queue
	self.privatechat = privatechat
	self.peerconns = peerconns
	self.frame = frame

	self.userslistctrl = UsersList(self, -1,style = wxLC_REPORT|wxLC_VIRTUAL|wxSUNKEN_BORDER)
	self.userslist = []
	self.users = {}

        self.note = wxStaticText(self, -1, "This list does not auto-update")
	self.update = wxButton(self, -1, "Update")
	
        sizerh = wxBoxSizer(wxHORIZONTAL)
        sizerh.Add(self.update,0,wxEXPAND)
        sizerh.Add(self.note,1,wxALIGN_CENTER)
        sizerv = wxBoxSizer(wxVERTICAL)
        sizerv.Add(sizerh,0,wxEXPAND)
        sizerv.Add(self.userslistctrl,1,wxEXPAND)
        self.SetSizer(sizerv)
        self.SetAutoLayout(true)

        EVT_BUTTON(self,self.update.GetId(), self.OnUpdate)

    def OnUpdate(self, event):
        self.queue.put(slskmessages.GlobalUserList())

    def setGlobalUsersList(self, msg):
        self.users = msg.users
        self.userslist = self.users.keys()
        self.userslist.sort()
        self.userslistctrl.SetItemCount(len(self.users))


class UsersList(wxListCtrl):
    """
    This is a list control for the users list in chat window and in global 
    users list. Gets users
    data from parent window (which should have a userslist attribute).
    """
    def __init__(self, parent, id, style = wxLC_REPORT|wxLC_VIRTUAL):
        wxListCtrl.__init__(self,parent,id,style = style)
        self.InsertColumn(0,"User", width=100)
        self.InsertColumn(1,"Speed",width=50)
        self.InsertColumn(2,"Files",width=50)
        self.SetItemCount(0)
        self.normal = wxListItemAttr()
        self.normal.SetTextColour("black")
        self.grey = wxListItemAttr()
        self.grey.SetTextColour("grey")

        self.imglist = wxImageList(18,18)
        self.online = self.imglist.Add(images.getOnlineBitmap())
        self.offline = self.imglist.Add(images.getOfflineBitmap())
        self.away = self.imglist.Add(images.getAwayBitmap())
        self.AssignImageList(self.imglist,wxIMAGE_LIST_SMALL)

        self.parent = parent

        self.menu = wxMenu()
        sendmessageID=wxNewId()
        self.menu.Append(sendmessageID, 'Send Message')
        EVT_MENU(self,sendmessageID, self.OnSendMessage)
        showipID=wxNewId()
        self.menu.Append(showipID, 'Show IP address')
        EVT_MENU(self,showipID, self.OnShowIp)
        getinfoID=wxNewId()
        self.menu.Append(getinfoID, 'Get User Info')
        EVT_MENU(self,getinfoID, self.OnGetInfo)
        browseID=wxNewId()
        self.menu.Append(browseID, 'Browse Files')
        EVT_MENU(self,browseID, self.OnBrowse)

        EVT_LIST_ITEM_RIGHT_CLICK(self, -1,self.OnRightClick)
        EVT_RIGHT_UP(self,self.OnRightUp)

    def OnRightClick(self,event):
        """ Saves name of a user in a lisr on a right-click."""
        id = event.GetIndex()
        self.SetItemState(id,wxLIST_STATE_FOCUSED,wxLIST_STATE_FOCUSED)
        self.focuseduser = self.GetItemText(id)

    def OnRightUp(self,event):
        """ Pops up a menu on a right-click in users list."""
        self.PopupMenu(self.menu, wxPoint(event.GetX(),event.GetY()))


    """ Handlers for the menu items"""
    def OnSendMessage(self, event):
        self.parent.privatechat.SendMessage(self.focuseduser)

    def OnShowIp(self, event):
        self.parent.queue.put(slskmessages.GetPeerAddress(self.focuseduser))

    def OnGetInfo(self, event):
        self.parent.frame.ProcessRequestToPeer(self.focuseduser, slskmessages.UserInfoRequest(None), self.parent.frame.userinfo)

    def OnBrowse(self, event):
        self.parent.frame.ProcessRequestToPeer(self.focuseduser, slskmessages.GetSharedFileList(None), self.parent.frame.userbrowse)


    def OnGetItemText(self, item, col):
        username =  self.parent.userslist[item]
        if col == 0:
            return username
        elif col == 1:
            return self.parent.users[username].avgspeed
        elif col == 2:
            return self.parent.users[username].files
        else:
            return None

    def OnGetItemAttr(self,item):
        username =  self.parent.userslist[item]
        if self.parent.users[username].slotsfull == 1:
            return self.grey
        else:
            return self.normal

    def OnGetItemImage(self,item):
        username = self.parent.userslist[item]
        if self.parent.users[username].status == 1:
            return self.away
        elif self.parent.users[username].status == 2:
            return self.online
        else:
            return -1

class ChatWindow(wxSplitterWindow):
    """
    The chat window that holds a users list, info window and dialog window.
    Also contains users dictionary that holds info about every user in a room.
    """
    def __init__(self, parent, id, queue, privatechat, peerconns, frame, style = wxNO_3D|wxSP_3D):
        wxSplitterWindow.__init__(self,parent,id,wxDefaultPosition,wxDefaultSize,style,"splitterWindow")
        splitter = wxSplitterWindow(self,-1,style=wxNO_3D|wxSP_3DSASH)
        self.userslistctrl = UsersList(self,-1,style=wxLC_REPORT|wxLC_VIRTUAL)
        self.SplitVertically(splitter,self.userslistctrl)
        self.SetSashPosition(400, true)
        self.info = OutputWindow(splitter,-1,style = wxTE_MULTILINE|wxTE_READONLY|wxTE_RICH)

        chatpanel = wxPanel(splitter,-1)
        self.chat = OutputWindow(chatpanel,-1,style = wxTE_MULTILINE|wxTE_READONLY|wxTE_RICH)
        self.mychatphrase = wxTextCtrl(chatpanel,-1, style = wxTE_PROCESS_ENTER)
        chatsizer = wxBoxSizer(wxVERTICAL)
        chatsizer.Add(self.chat,1,wxEXPAND)
        chatsizer.Add(self.mychatphrase,0,wxEXPAND)
        chatpanel.SetSizer(chatsizer)
        chatpanel.SetAutoLayout(true)

        splitter.SplitHorizontally(self.info,chatpanel)
        splitter.SetSashPosition(100,true)

        self.queue = queue
        self.privatechat = privatechat
        self.users = {} #without IP/fw cache
        self.peerconns = peerconns
        self.frame = frame

        EVT_TEXT_ENTER(self,self.mychatphrase.GetId(),self.OnEnter)

    def OnEnter(self, event):
        """ Processes chat phrase that we entered. """
        text = self.mychatphrase.GetLineText(0)
        self.queue.put(slskmessages.SayChatroom(self.room, text))
        self.mychatphrase.Clear()

    """ These functions process various messages from the server. """
    def UserJoinedRoom(self, msg):
        self.room = msg.room
        self.users[msg.username] = msg.userdata
        self.userslist = self.users.keys()
        self.userslist.sort()
        self.userslistctrl.SetItemCount(len(self.users))

        self.info.AppendText("* %s joined the room\n" %(msg.username))

    def JoinRoom(self, msg):
        self.room = msg.room
        self.users = msg.users
        self.userslist = self.users.keys()
        self.userslist.sort()
        self.userslistctrl.SetItemCount(len(self.users))

    def UserLeftRoom(self, msg):
        self.room = msg.room
        if msg.username in self.users.keys():
            del self.users[msg.username]
        self.userslist = self.users.keys()
        self.userslist.sort()
        self.userslistctrl.SetItemCount(len(self.users))
        self.info.AppendText("* %s left the room\n" %(msg.username))

    def SayChatRoom(self,msg):
        self.room = msg.room
        self.chat.AppendText("[%s] %s\n" %(msg.user,msg.msg))

    def QueuedDownloads(self,msg):
        self.users[msg.user].slotsfull = msg.slotsfull
        self.userslistctrl.SetItemCount(len(self.users))

    def GetUserStatus(self,msg):
        self.users[msg.user].status = msg.status
        if msg.status == 1:
            text = "* %s has gone away\n"
        else:
            text = "* %s has returned\n"
        self.info.AppendText(text %(msg.user))
        self.userslistctrl.SetItemCount(len(self.users))

    def GetUserStats(self,msg):
        self.users[msg.user].avgspeed = msg.avgspeed
        self.users[msg.user].downloadnum = msg.downloadnum
        self.users[msg.user].something = msg.something
        self.users[msg.user].files = msg.files
        self.users[msg.user].dirs = msg.dirs
        self.userslistctrl.SetItemCount(len(self.users))

class PrivateChatNotebook(wxNotebook):
    """
    This class is a notebook of private chat windows. Actual window 
    pointers are stored in user dictionary.
    """
    def __init__(self, parent, id, queue, style=wxCLIP_CHILDREN|wxNB_RIGHT):
        wxNotebook.__init__(self,parent,id,style = style)
        self.users={}
        self.queue = queue

    def ShowMessage(self,msg):
        """ Processes MessageUser message from the server. """
        if not self.users.has_key(msg.user):
            self.users[msg.user]=PrivateChatWindow(self, -1, msg.user)
            self.AddPage(self.users[msg.user],msg.user)
        self.users[msg.user].AddText("[%s] %s\n" %(msg.user, msg.msg))

    def SendMessage(self, user):
        """ Adds a notebook tab with a user's name on it, and prvate chat 
        window inside the tab
        """
        if not self.users.has_key(user):
            self.users[user]=PrivateChatWindow(self, -1, user)
            self.AddPage(self.users[user],user)

class PrivateChatWindow(wxPanel):
    """ This is a window for chatting with someone in private"""
    def __init__(self, parent, id, user):
        wxPanel.__init__(self, parent, id)
        self.chat = wxTextCtrl(self,-1,style = wxTE_MULTILINE|wxTE_READONLY|wxTE_RICH)
        self.mychatphrase = wxTextCtrl(self,-1, style = wxTE_PROCESS_ENTER)
        self.close = wxButton(self, -1, "Close")
        sizerh = wxBoxSizer(wxHORIZONTAL)
        sizerh.Add(self.mychatphrase,1,wxEXPAND)
        sizerh.Add(self.close,0,wxEXPAND)
        sizerv = wxBoxSizer(wxVERTICAL)
        sizerv.Add(self.chat,1,wxEXPAND)
        sizerv.Add(sizerh,0,wxEXPAND)

        self.SetSizer(sizerv)
        self.SetAutoLayout(true)
        EVT_BUTTON(self,self.close.GetId(), self.OnClose)
        EVT_TEXT_ENTER(self,self.mychatphrase.GetId(),self.OnEnter)

        self.user = user
        self.parent = parent


    def AddText(self,text):
        """ Rather obvious really """
        self.chat.AppendText(text)

    def OnClose(self, event):
        """ Close the window and update notebook's window list."""
        del self.parent.users[self.user]
        self.parent.DeletePage(self.parent.GetSelection())
        self.parent.SetSelection(0)

    def OnEnter(self, event):
        """ Sends our chat phrase and updates the window."""
        text = self.mychatphrase.GetLineText(0)
        self.AddText("> "+ text+ "\n")
        self.parent.queue.put(slskmessages.MessageUser(self.user, text))
        self.mychatphrase.Clear()

